package gregapi.block;

import gregapi.data.CS.ModIDs;
import gregapi.util.UT;
import ic2.api.crops.ICropTile;
import ic2.api.tile.IWrenchable;

import java.util.ArrayList;
import java.util.Arrays;

import net.minecraft.block.*;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.event.entity.player.UseHoeEvent;
import cpw.mods.fml.common.FMLLog;
import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.eventhandler.Event.Result;

/**
 * @author Gregorius Techneticies
 * 
 * For Internal Use.
 */
public class ToolCompat {
	private static boolean IC_WRENCHABLE = false, IC_CROPTILE = false, RC_INSTALLED = false, CHECK_ALL = true;
	
	private static void checkAvailabilities() {
		if (CHECK_ALL) {
			try {
				IWrenchable.class.getCanonicalName();
				IC_WRENCHABLE = true;
			} catch(Throwable e) {/**/}
			try {
				ICropTile.class.getCanonicalName();
				IC_CROPTILE = true;
			} catch(Throwable e) {/**/}
			RC_INSTALLED = Loader.isModLoaded(ModIDs.RC);
			CHECK_ALL = false;
		}
	}
	
	/** Providing compatibility for vanilla Blocks and certain Mod Interfaces. */
	public static long onToolClick(Block aBlock, String aTool, long aRemainingDurability, Entity aPlayer, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, World aWorld, byte aSide, int aX, int aY, int aZ, float aHitX, float aHitY, float aHitZ) {
		checkAvailabilities();
		
		byte aMeta = (byte)aWorld.getBlockMetadata(aX, aY, aZ);
		TileEntity aTileEntity = UT.Worlds.getTileEntity(aWorld, aX, aY, aZ, true);
		EntityPlayer aEntityPlayer = aPlayer instanceof EntityPlayer ? (EntityPlayer)aPlayer : null;
		EntityLivingBase aEntityLiving = aPlayer instanceof EntityLivingBase ? (EntityLivingBase)aPlayer : null;
		
		try {
		
		if (aTool.equals("hoe") && aEntityPlayer != null && aEntityPlayer.canPlayerEdit(aX, aY, aZ, aSide, aStack)) {
			UseHoeEvent event = new UseHoeEvent(aEntityPlayer, aStack, aWorld, aX, aY, aZ);
			if (!MinecraftForge.EVENT_BUS.post(event)) {
				if (event.getResult() == Result.ALLOW) return 10000;
				
				if (aSide != 0 && aBlock.isAir(aWorld, aX, aY, aZ) && (aBlock == Blocks.grass || aBlock == Blocks.dirt)) {
				    aWorld.playSoundEffect(aX + 0.5F, aY + 0.5F, aZ + 0.5F, Blocks.farmland.stepSound.getStepResourcePath(), (Blocks.farmland.stepSound.getVolume() + 1.0F) / 2.0F, Blocks.farmland.stepSound.getPitch() * 0.8F);
				    if (!aWorld.isRemote) aWorld.setBlock(aX, aY, aZ, Blocks.farmland);
					return 10000;
				}
			}
		}
		
		
		if (aWorld.isRemote) return 0;
		
		
		if (aTool.equals("sense") || aTool.equals("scythe")) {
	    	if (IC_CROPTILE && aTileEntity instanceof ICropTile) {
	    		int tDamage = 0;
	    		for (int i = -1; i < 2; i++) for (int j = -1; j < 2; j++) for (int k = -1; k < 2; k++) if ((aTileEntity = UT.Worlds.getTileEntity(aWorld, aX+i, aY+j, aZ+k, true)) instanceof ICropTile && ((ICropTile)aTileEntity).harvest(true)) tDamage += 10000;
	    		return tDamage;
	    	}
		}
		if (aTool.equals("igniter")) {
			if (aBlock instanceof BlockTNT) {
				((BlockTNT)aBlock).func_150114_a(aWorld, aX, aY, aZ, aMeta, aEntityLiving);
				return 10000;
			}
		}
		if (aTool.equals("rotator")) {
			if (aBlock.rotateBlock(aWorld, aX, aX, aX, ForgeDirection.getOrientation(aSide))) return 10000;
		}
		if (aTool.equals("screwdriver")) {
	    	if (aBlock instanceof BlockRedstoneDiode) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta / 4) * 4  + (((aMeta%4) + 1) % 4), 3)) return 10000;
	    	}
		}
		if (aTool.equals("crowbar")) {
			if (!RC_INSTALLED && aBlock instanceof BlockRailBase) {
				aWorld.isRemote = true;
				// WTF, why are the two Coordinate Parameters in isFlexibleRail switched? And then it is used like x y z instead of using the broken namings.
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, ((BlockRailBase)aBlock).isFlexibleRail(aWorld, aX, aY, aZ)?(aMeta + 1) % 10:((aMeta / 8) * 8) + (((aMeta%8)+1) % 6), 0);
				aWorld.isRemote = false;
		    	return tResult?10000:0;
			}
		}
		if (aTool.equals("softhammer")) {
	    	if (aBlock == Blocks.lit_redstone_lamp) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, Blocks.redstone_lamp, 0, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.redstone_lamp) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, Blocks.lit_redstone_lamp, 0, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.golden_rail) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, (aMeta + 8) % 16, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.activator_rail) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, (aMeta + 8) % 16, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock instanceof BlockRotatedPillar) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta + 4) % 12, 3)) return 10000;
	    	}
		    if (aBlock == Blocks.piston || aBlock == Blocks.sticky_piston || aBlock == Blocks.dispenser || aBlock == Blocks.dropper) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta+1) % 6, 3)) return 10000;
		    }
		    if (aBlock == Blocks.pumpkin || aBlock == Blocks.lit_pumpkin || aBlock == Blocks.furnace || aBlock == Blocks.lit_furnace || aBlock == Blocks.chest || aBlock == Blocks.trapped_chest) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, ((aMeta-1)%4)+2, 3)) return 10000;
		    }
		    if (aBlock == Blocks.hopper) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta+1)%6==1?(aMeta+1)%6:2, 3)) return 10000;
		    }
		}
		if (aTool.equals("wrench") || aTool.equals("monkeywrench")) {
			byte aTargetSide = UT.Code.getSideWrenching(aSide, aHitX, aHitY, aHitZ);
	        if (IC_WRENCHABLE && aTileEntity instanceof IWrenchable) {
	       		if (((IWrenchable)aTileEntity).wrenchCanSetFacing(aEntityPlayer, aTargetSide)) {
	           		((IWrenchable)aTileEntity).setFacing(aTargetSide);
	       			return 10000;
	       		}
	        	if (((IWrenchable)aTileEntity).wrenchCanRemove(aEntityPlayer)) {
	        		int tDamage = Math.max(10000, (int)(30000 / ((IWrenchable)aTileEntity).getWrenchDropRate()));
	        		ArrayList<ItemStack> tDrops = aBlock.getDrops(aWorld, aX, aY, aZ, aMeta, 0);
	        		ItemStack tOutput = ((IWrenchable)aTileEntity).getWrenchDrop(aEntityPlayer);
	        		
	        		if (aWorld.setBlockToAir(aX, aY, aZ)) {
		        		if (aWorld.rand.nextInt(tDamage) < aRemainingDurability) {
			            	for (ItemStack tStack : tDrops) {
			            		if (tOutput == null) {
			                		aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, tStack));
			            		} else {
			                		aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, tOutput));
			                		tOutput = null;
			            		}
			            	}
		        		}
		        		return tDamage;
	        		}
	        	}
	        }
	    	
	    	if (aBlock instanceof BlockRotatedPillar) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta + 4) % 12, 3)) return 10000;
	    	}
	    	
	    	if (aBlock instanceof BlockWorkbench || aBlock instanceof BlockBookshelf) {
	    		if (aWorld.setBlockToAir(aX, aY, aZ)) {
        			aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, new ItemStack(aBlock, 1, aMeta)));
		    		return 10000;
        		}
	    	}
	    	
	    	if (aMeta == aTargetSide) {
		    	if (aBlock instanceof BlockPumpkin || aBlock instanceof BlockPistonBase || aBlock instanceof BlockDispenser || aBlock instanceof BlockFurnace || aBlock instanceof BlockChest || aBlock instanceof BlockHopper || aBlock instanceof BlockEnderChest) {
	        		if (aWorld.setBlockToAir(aX, aY, aZ)) {
	        			aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, new ItemStack(aBlock, 1, 0)));
			    		return 10000;
	        		}
		    	}
	    	} else {
		    	if (aBlock instanceof BlockPistonBase || aBlock instanceof BlockDispenser) {
					if (aMeta < 6 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
		    	if (aBlock instanceof BlockPumpkin || aBlock instanceof BlockFurnace || aBlock instanceof BlockChest || aBlock instanceof BlockEnderChest) {
					if (aTargetSide > 1 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
		    	if (aBlock instanceof BlockHopper) {
					if (aTargetSide != 1 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
	    	}
	    	if (Arrays.asList(aBlock.getValidRotations(aWorld, aX, aY, aZ)).contains(ForgeDirection.getOrientation(aTargetSide))) {
	    		if (aBlock.rotateBlock(aWorld, aX, aY, aZ, ForgeDirection.getOrientation(aTargetSide))) return 10000;
	    	}
		}
		
		} catch(Throwable e) {
			FMLLog.severe("Exception occured when ToolCompat was used at the Coordinates: [%d;%d;%d] at '%s' with TileEntity '%s' using the Tool '%s' %s", aX, aY, aZ, aBlock.getUnlocalizedName(), aTileEntity.getClass(), aTool, e.toString());
		}
		return 0;
	}
}