package gregapi.tileentity;

import gregapi.random.IHasWorldAndCoords;
import net.minecraft.entity.player.EntityPlayer;

/**
 * @author Gregorius Techneticies
 */
public interface ITileEntity extends IHasWorldAndCoords, ITileEntityUnloadable {
	/**
	 * @return the Mod of this Block for GUI purposes.
	 */
	public Object getModForGUI();
	public Object getGUIClient(int aGUIID, EntityPlayer aPlayer);
	public Object getGUIServer(int aGUIID, EntityPlayer aPlayer);
	
    /**
     * Sends a Block Event to the Client TileEntity, the byte Parameters are only for validation as Minecraft doesn't properly write Packet Data.
     */
    public void sendBlockEvent(byte aID, byte aValue);
    
	/**
	 * @return the amount of Time this TileEntity has been loaded.
	 */
	public long getTimer();
	
	/**
	 * Opens the GUI with this ID of this TileEntity
	 */
	public boolean openGUI(EntityPlayer aPlayer, int aID);
	
	/**
	 * Opens the GUI with the ID = 0 of this TileEntity
	 */
	public boolean openGUI(EntityPlayer aPlayer);
	
    /**
     * YOU MUST HAVE THIS INSIDE YOUR BLOCK CODE!!!
     * 
     * public void onNeighborChange(IBlockAccess aWorld, int aX, int aY, int aZ, int aTileX, int aTileY, int aTileZ) {
     *     TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
     *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aTileX, aTileY, aTileZ);
     * }
     * 
	 * public void onNeighborBlockChange(World aWorld, int aX, int aY, int aZ, Block aBlock) {
	 * 	   TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
	 *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aX, aY, aZ);
     * }
     */
	public void onAdjacentBlockChange(int aTileX, int aTileY, int aTileZ);
}