package gregapi.tileentity;

import gregapi.network.INetworkHandler;
import gregapi.network.IPacket;

/**
 * @author Gregorius Techneticies
 * 
 * TileEntity with Network Code
 */
public abstract class TileEntityBase3 extends TileEntityBase2 {
	/** Variable for updating Data to the Client */
	private boolean mSendClientData = false;
	
	/** @return the used Network Handler */
	public abstract INetworkHandler getNetworkHandler();
	
	/** @return a Packet containing all Data which has to be synchronised to the Client */
	public abstract IPacket getClientDataPacket(boolean aSendAll);
	
	/** sends all Data to the Clients in Range */
	public void sendClientData(boolean aSendAll) {getNetworkHandler().sendToAllPlayersInRange(getClientDataPacket(aSendAll), worldObj, xCoord, zCoord);}
	
	/** Called to send all Data to the close Clients */
	public void updateClientData() {mSendClientData = true;}
	
	@Override public void onCoordinateChange() {super.onCoordinateChange(); updateClientData();}
	
	@Override public net.minecraft.network.Packet getDescriptionPacket() {updateClientData(); return null;}
    
    @Override
    public final void updateEntity() {
    	if (mTimer == 0) {
    		markDirty();
    		onTickFirst();
    	}
    	onTickStart(mTimer);
    	super.updateEntity();
    	onTick(mTimer);
    	if (isServerSide() && (mSendClientData || onTickCheck(mTimer))) {
    		sendClientData(mSendClientData);
    		mSendClientData = false;
    	}
    	onTickEnd(mTimer);
    }
    
    /** The very first Tick happening to this TileEntity */
    public void onTickFirst() {/**/}
    
    /** The first Part of the Tick. */
    public void onTickStart(long aTimer) {/**/}
    
    /** The regular Tick. */
    public void onTick(long aTimer) {/**/}
    
    /** Use this to check if it is required to send an update to the Clients. */
    public boolean onTickCheck(long aTimer) {return false;}
    
    /** The absolutely last Part of the Tick. */
    public void onTickEnd(long aTimer) {/**/}
}